#!/bin/sh
# Copyright (C) 2007-2008 PeerTV Inc.
# All rights reserved.
#
# @file scroll.sh
# @brief Shell script which make upgrade from USB DOK. 
#        When HD Format option is selected from TV-Setup/Web Management the
#        HD Format is performed prior to upgrade itself.
#
# @notes The script is also used to fix the following issues if required:
#        * Network EEPROM fix for PeerStation based on EM8622 chip.
#        * WIFI Card Comex fix for PeerStation based on EM8622 chip.
#
# @author Baruch Shapira <baruch@peertv.com>
# @date 15-04-2008
 
. /etc/profile

if [ -z "$HDD_DEVICE" ]; then
        if [ -e /proc/ide/hda ]; then
		HDD_DEVICE=/dev/hda
	else
		HDD_DEVICE=/dev/hdb
	fi
fi

# Turn on debug mode
set -x

appsbin_path=$USB_PATH/utils

### Network eeprom fix - Start
if [ "$SIGMA22" = "YES" ]; then

	iobase=`grep 10ec8139 /proc/bus/pci/devices | cut -f 4`

	if [ -n "$iobase" ]; then
		   insmod $appsbin_path/8139too-eeprom.o iobase=0x$iobase
	fi
fi
### Network eeprom fix - End

### Wifi Comex fix - Start
ath_tool=$appsbin_path/ath_info
if [ -f /proc/bus/pci/00/03.0 -a -f $ath_tool ]; then
   subvid=`hexdump /proc/bus/pci/00/03.0 | egrep "^0+20 " | cut -d ' ' -f 8`
   subdid=`hexdump /proc/bus/pci/00/03.0 | egrep "^0+20 " | cut -d ' ' -f 9`
   if [ "$subvid" = "168c" ]; then
      modprobe wlan_scan_sta
      modprobe wlan_wep
      modprobe ath_rate_sample
      modprobe ath_pci
      iomem=`cat /proc/iomem | grep ' ath$' | cut -d '-' -f 1`
      if [ -n "$iomem" ]; then
         ifconfig ath0 up
         sleep 3
         regdomain=`$ath_tool $iomem | grep "Regulatory Domain:" | sed -r 's/.*:[ \t]+//g'`
         if [ "$regdomain" != "0x60" ]; then
            $ath_tool -f -w $iomem regdomain 0x60
         else
            echo regdomain is already set
         fi
      fi
      ifconfig ath0 down
      rmmod ath_pci
      rmmod ath_rate_sample
      rmmod wlan_wep
      rmmod wlan_scan_sta
      rmmod ath_hal
      rmmod wlan
   fi
fi
### Wifi Comex fix - End

upgrade_package_file_name=upgrade-6.0.7TI_sig22.tar.gz

upgrade_eng_path=/bin/upgrade
export PATH=$PATH:$upgrade_eng_path
alt_browser=0
fatal_error=0

# check whether all required binary tools are found on USB DOK
MIN_DOK_FILES_LIST="mke2fs osdbuf_control sfdisk std_partitions"
for item in $MIN_DOK_FILES_LIST ; 
do
	if [ -f $appsbin_path/$item ]; then
		echo "Utility $item found, OK" 
	else
		echo "ERROR: $item binary not found!"
		exit 0
	fi
done

if [ "$SIGMA22" = "YES" ]; then
        insmod llad.o
        insmod em8xxx.o
        insmod mambolfb.o
fi

# must be read before mounting cfgfs !
if [ "$SIGMA34" = "YES" ]; then
	rootfs_mrua_version=`cat /etc/mipsutils_version`
else
        rootfs_mrua_version=`cat /etc/armutils_version`
fi

# userfs integrity test
$upgrade_eng_path/chkufs.sh
if [ $? = 0 ]; then
	# mounting SYSTEM_PATH
	mtd_block_num=`cat /proc/upgrade/userfs_num`
	mount -t cramfs -r /dev/mtdblock$mtd_block_num $SYSTEM_PATH

	# must be read before mounting cfgfs !
	if [ "$SIGMA34" = "YES" ]; then
        	userfs_mrua_version=`cat /system/mipsutils_version`
	else
        	userfs_mrua_version=`cat /system/armutils_version`
	fi

	# Verifying MRUA version rootfs/userfs competability 
	if [ "$rootfs_mrua_version" != "$userfs_mrua_version" ]; then
		echo "MRUA ver. of rootfs&userfs differ"
		echo "MRUA ver. of rootfs&userfs differ" > /tmp/usbd_osd

		alt_browser=1
	fi
else
	alt_browser=1
	echo "USERFS is damaged !"
fi

#initialize the display according to selected video display mode
displaysettings.upg

if [ $alt_browser -eq 0 ]; then
	# Loading browser
	makesettings.upg
else
	# Loading alt. browser
	$upgrade_eng_path/ascii_browser 0 -X 50 -Y 50 -S 2 -F 255,255,255,255 -B 0,0,255,255 -T 1 $upgrade_eng_path/font.psf " " &
fi

# getting the upgrade code
last_error_code=`cat /proc/upgrade/pending`
echo 0 > /proc/upgrade/pending
echo 0 > /tmp/usbd_status_num
echo 0 > /tmp/usbd_status_num_max
echo 0 > /tmp/usbd_pb_total
echo 0 > /tmp/usbd_pb_cur

# calculating overall stages number for different scenarios
if [ "$SIGMA22" = "YES" ]; then
	tmp_upgrade_with_hd_stages=30
	tmp_upgrade_without_hd_stages=22
	tmp_hd_check_stages=3
	tmp_hd_format_stages=33
	tmp_hd_clear_data_stages=1	
else
        tmp_upgrade_with_hd_stages=36
        tmp_upgrade_without_hd_stages=28
        tmp_hd_check_stages=3
        tmp_hd_format_stages=39
        tmp_hd_clear_data_stages=1
fi

# Format HD before upgrade if requested
if  [ $last_error_code = 250 ]; then
	echo "Formatting HD"
        modprobe ext3
        if [ !-e /proc/ide/hda -a !-e /proc/ide/hdb ]; then
                # Requesting reboot
                echo "Locating HDD" > /tmp/usbd_osd
                echo "-126|HDD not found - Replace the HDD" > /tmp/usbd_error
                echo "-126|HDD not found - Replace the HDD"
		echo "ERROR" > /tmp/usbd_action

                echo "Remove dok and wait for reboot"
                while [ 0 = 0 ]; do
                        grep usb /proc/hardware > /dev/null 2>&1
                        if [ $? -ne 0 ]; then
                                echo "no usb attached"
                                /sbin/reboot
                                exit 255
                        fi
                        sleep 2
                done
                exit 0
        fi

        echo  "DISK_FORMAT" > /tmp/usbd_action
        echo $tmp_hd_format_stages > /tmp/usbd_status_num_max

        sleep 3;
        $appsbin_path/sfdisk --force -uM $HDD_DEVICE < $appsbin_path/std_partitions
        $appsbin_path/mke2fs -j -b 4096 $MNT_BLKDEV
        if [ "`cat /tmp/usbd_action`" = "DISK_FORMAT_ERROR" ]; then
        	echo "Remove dok and wait for reboot"
                while [ 0 = 0 ]; do
                	grep usb /proc/hardware > /dev/null 2>&1
                        if [ $? -ne 0 ]; then
                        	echo "no usb attached"
                                /sbin/reboot
                                exit 255
                        fi
                     	sleep 2
                done
                exit 0
        fi

        $appsbin_path/mke2fs -j $SYS_BLKDEV
        if [ "`cat /tmp/usbd_action`" = "DISK_FORMAT_ERROR" ]; then
                echo "Remove dok and wait for reboot"
                while [ 0 = 0 ]; do
                        grep usb /proc/hardware > /dev/null 2>&1
                        if [ $? -ne 0 ]; then
                                echo "no usb attached"
                                /sbin/reboot
                                exit 255
                        fi
                        sleep 2
                done
                exit 0
	fi

        $appsbin_path/mke2fs -j $PRV_BLKDEV
        if [ "`cat /tmp/usbd_action`" = "DISK_FORMAT_ERROR" ]; then
                echo "Remove dok and wait for reboot"
                while [ 0 = 0 ]; do
                        grep usb /proc/hardware > /dev/null 2>&1
                        if [ $? -ne 0 ]; then
                                echo "no usb attached"
                                /sbin/reboot
                                exit 255
                        fi
                        sleep 2
                done
                exit 0
        fi
	#rm /tmp/usbd_status_num_max
        echo  "RUNNING" > /tmp/usbd_action
# Mounting HD if any
elif [ -e /proc/ide/hda -o -e /proc/ide/hdb ]; then
   modprobe ext3

   hd_dev_1=`$appsbin_path/sfdisk "$HDD_DEVICE"1 -s`
   if [ $? = 0 ]; then 
   	hd_dev_1=`expr $hd_dev_1 '/' 1024`
   else
   	hd_dev_1=0
   fi
   hd_dev_2=`$appsbin_path/sfdisk "$HDD_DEVICE"2 -s`
   if [ $? = 0 ]; then
        hd_dev_2=`expr $hd_dev_2 '/' 1024`
   else
        hd_dev_2=0
   fi
   hd_dev_3=`$appsbin_path/sfdisk "$HDD_DEVICE"3 -s`
   if [ $? = 0 ]; then 
	hd_dev_3=`expr $hd_dev_3 '/' 1024`
   else
        hd_dev_3=0
   fi
   size_offset_mb=100

   # HD exists - checking partitioning
   par_num=1
   par_ok=1
   for x in `cat $appsbin_path/std_partitions | cut -f 2 -d ','`
   do
	check=0
	if [ "$x" != ";" -a "$x" != "EOF" ]; then
		case $par_num in
			1)	par_size=$hd_dev_1;;
			2)	par_size=$hd_dev_2;;
			3)	par_size=$hd_dev_3;;
			4)	par_size=$hd_dev_4;;
			*)	check=1;;
		esac 
		
		# Checking whether partition size is within rang
		if [ "$par_size" -le "`expr $x + $size_offset_mb`" -a  "$par_size" -ge "`expr $x - $size_offset_mb`" ]; then
			check=1
		fi
		
		if [ $check -eq 1 ]; then
			echo "Partition $par_num geometry is correct"
		else
			echo "Partition $par_num geometry is wrong"
			par_ok=0
		fi 
	fi
	par_num=`expr $par_num + 1`
   done

   # If bad partitioning was found
   if [ $par_ok = 0 ]; then
   	echo "Bad partitioning"

	# first partition is not exists -> new HD -> format
	if [ $hd_dev_1 -eq 0 ]; then
		echo  "DISK_FORMAT" > /tmp/usbd_action
		echo $tmp_hd_format_stages > /tmp/usbd_status_num_max

                sleep 1;
                $appsbin_path/sfdisk --force -uM $HDD_DEVICE < $appsbin_path/std_partitions
                sleep 1;
                $appsbin_path/mke2fs -j -b 4096 $MNT_BLKDEV
		if [ "`cat /tmp/usbd_action`" = "DISK_FORMAT_ERROR" ]; then
			echo "Remove dok and wait for reboot"
			while [ 0 = 0 ]; do
				grep usb /proc/hardware > /dev/null 2>&1
				if [ $? -ne 0 ]; then
					echo "no usb attached"
					/sbin/reboot
					exit 255
				fi
				sleep 2
			done
        	        exit 0
		fi

                $appsbin_path/mke2fs -j $SYS_BLKDEV
		if [ "`cat /tmp/usbd_action`" = "DISK_FORMAT_ERROR" ]; then
			echo "Remove dok and wait for reboot"
			while [ 0 = 0 ]; do
				grep usb /proc/hardware > /dev/null 2>&1
				if [ $? -ne 0 ]; then
					echo "no usb attached"
					/sbin/reboot
					exit 255
				fi
				sleep 2
			done
        	        exit 0
		fi

                $appsbin_path/mke2fs -j $PRV_BLKDEV
		if [ "`cat /tmp/usbd_action`" = "DISK_FORMAT_ERROR" ]; then
			echo "Remove dok and wait for reboot"
			while [ 0 = 0 ]; do
				grep usb /proc/hardware > /dev/null 2>&1
				if [ $? -ne 0 ]; then
					echo "no usb attached"
					/sbin/reboot
					exit 255
				fi
				sleep 2
			done
        	        exit 0
		fi
	        #rm /tmp/usbd_status_num_max
	else
		# Requesting reboot
		echo "Unexpected Partition Scheme - Format or Replace HD" > /tmp/usbd_osd
		echo 100 > /tmp/usbd_pb_total
		echo 100 > /tmp/usbd_pb_cur
		sleep 1
		echo "DISK_SCHEME_MISMATCH" > /tmp/usbd_action

		# Waiting for reboot
		echo "Remove dok and wait for reboot"
		while [ 0 = 0 ]; do
	        grep usb /proc/hardware > /dev/null 2>&1
	        if [ $? -ne 0 ]; then
	                echo "no usb attached"
	                /sbin/reboot
	                exit 255
	        fi
	        sleep 2
		done
		exit 0
	fi
   else
	#simple upgrade with valid Hard Disk
	echo $tmp_upgrade_with_hd_stages > /tmp/usbd_status_num_max
   fi

   if [ -z "$SYS_BLKDEV" -o -z "$MNT_BLKDEV" ]; then 
   	SYS_BLKDEV=$HDD_DEVICE3
	MNT_BLKDEV=$HDD_DEVICE1
   fi

#   cat /proc/mounts
#   mkdir -p $MNT_PATH $EXT_SYS /tmp/var/locks $USB_PATH $DEVICE_PATH/hd
#   mount -t ext3 -o data=journal,commit=1,rw $SYS_BLKDEV $EXT_SYS
#   mount -t ext3 -o data=journal,commit=1,rw $PRV_BLKDEV $PRV_SYS
#   mount -t ext3 -o data=journal,commit=1,rw $MNT_BLKDEV $MNT_PATH
#   ln -sfn $MNT_PATH $DEVICE_PATH/hd
else
	# simple upgrade is initiated without Hard Disk
	echo $tmp_upgrade_without_hd_stages > /tmp/usbd_status_num_max
fi

sleep 2

# Finding last error code
last_error_code=`cat /proc/upgrade/pending`
if [ $last_error_code -ge 250 -o $last_error_code = 0 ]; then
	echo "Upgrade started !"
else
	# Recovery
	echo "Upgrade was interrupted - restoring !"
	echo "Last upgrade aborted with ($last_error_code)" > /tmp/usbd_error
fi

# Initiating upgrade
$upgrade_eng_path/store_settings.sh store

# Enabling reset button
echo "ready" > /proc/upgrade/pushb

$upgrade_eng_path/upgrade_eng -s $USB_PATH/$upgrade_package_file_name -v 5
if [ $? = 0 ] ; then
        if [ ! -e back_to_defaults ]; then
		if [ ! -e back_to_defaults ]; then 
		$upgrade_eng_path/store_settings.sh ret; 
	fi
	else
		# Reset Video Display Mode control in EEPROM
		echo 0 > /proc/upgrade/vmode
	fi
        if [ -e /proc/ide/hda -o -e /proc/ide/hdb ]; then
		$upgrade_eng_path/upgrade_hd.sh $USB_PATH/$upgrade_package_file_name $upgrade_eng_path 
                if [ $? != 0 ] ; then
                        echo "Upgrade HD FAILED!"
                        echo "ERROR" > /tmp/usbd_action
                fi
	else
		echo "HD (/proc/ide/hda or /proc/ide/hdb) was not found !"
	fi
else
	echo "ERROR" > /tmp/usbd_action
        echo "Upgrade FAILED !"
fi

umount -r $SYS_BLKDEV
umount -r $PRV_BLKDEV
umount -r $MNT_BLKDEV

echo "DONE! Please remove the disk on key and wait for system to reboot"

# Requesting reboot
echo "Remove dok and wait for reboot" > /tmp/usbd_osd
echo 100 > /tmp/usbd_pb_total
echo 100 > /tmp/usbd_pb_cur
sleep 1
echo "DONEUSB" > /tmp/usbd_action

# Waiting for reboot
echo "Remove dok and wait for system to reboot"
while [ 0 = 0 ]; do
	grep usb /proc/hardware > /dev/null 2>&1
	if [ $? -ne 0 ]; then
		echo "no usb attached"
		/sbin/reboot
                exit 255
	fi
	sleep 2
done
exit 0
